
/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   : UG_LawnMaker_mytest_01.pov
 Version     : 3.8
 Description : Example file to show usage of LawnMaker 1.2 by Uwe Glei
               Example of a single patch over a height_field.
               
 Date        : November 2004 / October 2020
 Author      : Thomas de Groot
 E-mail      : 
 
 Copyright (C) 2004/2020. All rights reserved.
**********************************************************************************/

//+w640 +h450 +a0.3 +am2 +bm2 +bs8 +wt6

#version 3.8;

#include "rand.inc"

global_settings {
  assumed_gamma 1.0
  max_trace_level 5
  radiosity {
    pretrace_start 0.08           // start pretrace at this size
    pretrace_end   0.04           // end pretrace at this size
    count 50, 1000                // higher -> higher quality (1..1600) [35]
    nearest_count 10, 5           // higher -> higher quality (1..10) [5]
    error_bound 1                 // higher -> smoother, less accurate [1.8]
    recursion_limit 2             // how much interreflections are calculated (1..5+) [2]
    low_error_factor .3           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    maximum_reuse 0.1							// new parameter 3.7 [0.2]
    brightness 1                  // brightness of radiosity effects (0..1) [1]

    adc_bailout 0.01/2
    normal off                   // take surface normals into account [off]
    media off                    // take media into account [off]
    always_sample off           // turn sampling in final trace off [on]
    //max_sample 1.0              // maximum brightness of samples
  }
}

sphere {0, 1e4 pigment {rgb 0.5} finish {emission 1.0 diffuse 0.0}}
plane {y, -0.5 pigment {rgb 0.5}}

camera { 
  location  <0, 10, -20> 
  direction 1.5*z
  right     x*image_width/image_height
  angle     50
  look_at   <0.0, 0.0, 0.0>
}

light_source {
  <-30, 30, -30>            
  color rgb <0.98, 0.98, 0.97>*3 
}

light_source {
  <30, 30, 30>            
  color rgb <1, 1, 1>*0.1 
  shadowless
}

//==================================================================================
#include "DrawTheAxes.inc"
#include "lawnmaker.inc"
#include "Halm1.inc"

//==================================================================================
// A height_field on which to plant the grass:

#declare HFscale = <15, 2, 15>;

#declare HF =
#difference {
  intersection {
    //height_field {png "Spherical_01.png" gamma 1.0
    plane {y, 0
      texture {
        pigment {
          //image_map {png "Spherical_01.png" gamma srgb map_type 0 interpolate 2 once}
          image_map {png "WildGrass_2_UR_512.png" gamma srgb map_type 0 interpolate 2 once}
          //image_map {png "WildGrass_2_UR_512_Height.png" gamma srgb map_type 0 interpolate 2 once}
          rotate x*90 // lay X-Y image map down onto X-Z height field plane
        }
      }
	    translate <-0.5, 0.0, -0.5>
	  }
	  box {<-0.499, -1.0, -0.499>, <0.499, 1.0, 0.499> pigment {srgb <0.6, 0.5, 0.2>}}
    scale HFscale
  }
  plane {y, -0.5 pigment {rgb 0}}
}

//==================================================================================
//==================================================================================
// How to use Lawnmaker (Only the necessary values are changed)

//---------------------------------------------------------------------------------(1)
// Size and density of the grass patch:

//#declare LawnStart = <-9, -20.5>;
//#declare LawnStop  = < 9,  20.5>;
#declare LawnStart = <-7.4, -7.4>;
#declare LawnStop  = < 7.4,  7.4>;
#declare LawnStepX = <0.075, 0.075>;
#declare LawnStepZ = <0.075, 0.075>;

//---------------------------------------------------------------------------------(2)
// Parameter controlling the individual sizes of the grass blades (including their absence):

#local Var = 1;

#switch (Var)
#case (1)
// A first variation:
#declare Map = pigment {image_map { png "WildGrass_2_UR_512_Height.png" gamma srgb} rotate 90*x translate <-0.5, 0.0, -0.5> scale <HFscale.x*0.01, 1.0, HFscale.z*0.01> }
#declare LawnHeight = 
pigment {
  pigment_pattern {
    boxed //cylindrical
    pigment_map {
      [0.3 rgb 1]
      [0.7 rgb 0]
    }
    //warp {turbulence 0.6}
    scale 10
  }
  pigment_map {
    [0.0 rgb 1]
    [0.5 Map]
    [0.5 rgb 0]
  }
  scale 1
}

#break
#case (2)
// A second variation:
#declare Map = pigment {image_map { png "Spherical_01.png" gamma srgb} rotate 90*x translate <-0.5, 0.0, -0.5> scale HFscale }
#declare LawnHeight = 
pigment {
  pigment_pattern {Map}
  pigment_map {
    [0.01 rgb 0]
    [1.0 rgb 1]
  }
}

#end

//---------------------------------------------------------------------------------(3)
// Parameters allowing positioning on a height_field (Variation 1):

#switch (Var)
#case (1)
#declare LawnPosStretch = HFscale.y;
#declare LawnPosition   = pigment { image_map { png "Spherical_01.png" gamma srgb} rotate 90*x translate <-0.5, 0.0, -0.5> scale HFscale }
#end

//---------------------------------------------------------------------------------(4)
// For LawnDirection and LawnInclination, see lawnmaker.inc
// Otherwise, TypeTurn and TypeIncline below, control the blades.

//---------------------------------------------------------------------------------(5)
// New color map used for the straws:

#declare LawnColor[4] =
pigment {
  spotted turbulence 2 scale 0.5
  colour_map {
    [0.0 rgb <0.95, 0.7, 0.25>]
    [1.0 rgb <0.9, 0.8, 0.3>]
  }
}
 
//---------------------------------------------------------------------------------(6)
// Modelling of the grass patch:

// Suggestion: to save space in the list below, use e.g. #declare h = <0.2,0.2,0.5>;

#declare T = 6;
#declare TypeGeometry = array[T] { Blade1,     Blade2,     Blade3,     Blade4,     Blade5,     Halm1   }
#declare TypeWeight   = array[T] { 4,          4,          4,          3,          3,          0.1     }
#declare TypeSize1    = array[T] { <.7,.6,.7>, <.7,.5,.7>, <.6,.4,.6>, <.6,.4,.6>, <.6,.4,.6>, <1,1,1> }
#declare TypeSize2    = array[T] { <1,1.5,1>,  <1,1.4,1>,  <1,1.2,1>,  <1,1.1,1>,  <1,1.1,1>,  <3,3,3> }
#declare TypeColMap   = array[T] { 0,          0,          1,          2,          2,          4       }
#declare TypeTurn     = array[T] { <0, 360>,   <0, 360>,   <0, 360>,   <0, 360>,   <0, 360>,   <0, 360>}
#declare TypeIncline  = array[T] { <0, 50>,    <0, 30>,    <0, 10>,    <0, 5>,     <-2, 2>,    <-5, 5> }

//---------------------------------------------------------------------------------(7)
// invoking LawnMaker and thus actually producing the grass:

LawnMaker()

//---------------------------------------------------------------------------------(8)
// And here are all the scene objects together; GrassPatch is the grass, obviously ;-)

union {
  object {MyAxis(6, 4, 6, 1) scale 0.05 translate 1*y}
  object {GrassPatch}
  object {HF}
  rotate 20*y
}


